==> gm-1.6.3_Linux.tar.gz: gm-1.6.3_Linux/tests/gm_counters.c <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation                      *
 * Copyright (c) 1999 by Myricom, Inc.                                   *
 * All rights reserved.  See the file `COPYING' for copyright notice.    *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#if defined __STDC__ || defined WIN32
#if GM_OS_VXWORKS
#include "vxWorks.h"
#endif  /* GM_OS_VXWOKRS */
#include <stdarg.h>
#else
#include <varargs.h>
#endif

/* This program uses undocumented software interfaces that are not
   intended for use in code written outside Myricom.  Hence the
   following undocumented includes. */

#include "gm_internal.h"
#include "gm_enable_error_counters.h"
#include "gm_enable_debug_counters.h"
#include "gm_enable_packet_counters.h"

/* this 1024 was hard-coded in drivers/gm.c */
/* don't really want to get all the data from a card with 2Meg+ */
#define MY_MAX_NODE	(3*1024)

/****************************************************************
 * global data
 ****************************************************************/

int board_num = 0,
    port_num  = 0;

#if	GM_OS_AIX4

struct gm_lanai_globals *globals = NULL;

#else	/* GM_OS_AIX4 */

unsigned int space[(sizeof(struct gm_lanai_globals) +
		       ((MY_MAX_NODE+1) * sizeof(struct gm_connection))) / sizeof(int)];

struct gm_lanai_globals *globals = (struct gm_lanai_globals *) space;

#endif	/* GM_OS_AIX4 */

gm_u64_t current_rtc;

gm_port_t *port;

extern int atoi(const char *);

/****************************************************************
 * Command line processing
 ****************************************************************/

#define TP(a) fprintf (stderr, a)

static void
print_synopsis(void)
{
  TP ("\n");
  TP ("Myricom 'gm_counters' displays diagnostic statistics about the\n");
  TP ("operation of the Myrinet interface.  This program is used almost\n");
  TP ("exclusively for problem diagnosis.  The output from 'gm_counters'\n");
  TP ("is a subset of the output from 'gm_debug'.\n");
}


static void
print_options (void)
{
  TP ("\nusage: gm_counters [options]\n\n");

  TP ("-B, --board=NUM      [0] board from which to display the counters\n");
  TP ("-p, --port=NUM       [0] port number to use\n");
  TP ("    --help               output usage and options information, then exit\n");
  printf ("\n");

#undef TP
}


static void
#ifdef va_dcl
usage (fmt, va_alist)
     const char *fmt;
     va_dcl
#else
usage (const char *fmt, ...)
#endif
{
  va_list ap;

  print_options();

/* Now print the caller's diagnostic */
#if defined __STDC__ || defined WIN32
  va_start (ap, fmt);
#else
  va_start (ap);
#endif
  vfprintf (stderr, fmt, ap);
  va_end (ap);

} /* usage() */


static void
parse_args (int argc, char *argv[])
{
/* NOTE: If you change the operating characteristics or the flags
 *       used by this program, please notify Max so he can change the
 *       Myrinet Handbook document.
 */
  int i;

  /* The 'F' macro is for all flags with no argument (parameter)     */
#define F(name,action) if (strcmp (argv[i], name) == 0) { action; continue; }

  for (i = 1; i < argc; i++)
  {
      /* Dispose of the --help flag first                                */
      if (strcmp (argv[i], "--help") == 0)
	  {
		  print_synopsis();
		  print_options();
		  gm_exit (GM_SUCCESS);
	  }

      /* Here are the flags with no argument.                            */
      /* Please keep short form, if any, after its long form equivalent, */
	  /* but keep the long form options in alpha sequence.               */

      /* None in gm_counters at present */

#undef F

      /* The check_opt macro is for long-form options with numeric        */
      /* arguments and simple validation tests.                           */
#define check_opt(prefix, var, test)							\
	  if (sscanf (argv[i], "--" prefix "=%i", &var) == 1)		\
	  {															\
	      if (!(test))											\
		  {														\
			  usage ("bad value for option \"%s\"\n", argv[i]);	\
			  gm_exit (GM_FAILURE);								\
		  }														\
	      continue;												\
	  }

      /* Here are the flags with arguments, sequenced as above */

      check_opt ("board", board_num, board_num >= 0);
      if (strcmp (argv[i], "-B") == 0)
	  {
		  if (i + 1 >= argc)
		  {
			  usage ("Board number expected after '-B'.\n");
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  board_num = atoi (argv[++i]);
		  if (board_num < 0)
		  {
			  usage ("bad board number: %d\n", board_num);
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  continue;
	  }

      /* Note that at present the parameter to gm_num_ports is a dummy */
      check_opt ("port", port_num,
				 (port_num >= 0 && port_num < gm_num_ports ((gm_port_t *)0)));
      if (strcmp (argv[i], "-p") == 0)
	  {
		  if (i + 1 >= argc)
		  {
			  usage ("Port number expected after '-p'.\n");
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  port_num = atoi (argv[++i]);
		  if (port_num < 0 || port_num >= gm_num_ports ((gm_port_t *)0))
		  {
			  usage ("bad port number: %d\n", port_num);
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  continue;
	  }

#undef check_opt

      usage ("unknown arg: '%s'\n", argv[i]);
      gm_exit (GM_FAILURE);
  }
}



/****************************************************************
 * produce the actual output
 ****************************************************************/

static int
gm_counters(int argc, char **argv)
{
	int length;
	gm_status_t status;

	parse_args(argc, argv);

	printf("gm_counters: Opening board %d port %d\n", board_num, port_num);
	fflush(stdout);				/* just in case we hang in the open */

	status = gm_open (&port, board_num, port_num, "gm_counters",
					  (enum gm_api_version) GM_API_VERSION);
	if (status != GM_SUCCESS) {
		gm_perror("gm_counters: _gm_open_common failed", status);
		gm_exit(status);
	}

	length = sizeof(struct gm_lanai_globals) +
	    (MY_MAX_NODE * sizeof(struct gm_connection));

#if	GM_OS_AIX4
	if ((globals = malloc (length)) == NULL) {
		printf ("gm_counters: error in malloc\n");
		gm_exit (GM_FAILURE);
	}
#endif	/* GM_OS_AIX4 */

	status = _gm_get_globals(port, (gm_u8_t *) globals, length);
	if (status != GM_SUCCESS) {
		gm_perror ("failed to get LANai globals", status);
		gm_exit(status);
	}

	/* now we have a copy of the lanai_globals */

	if (GM_ENABLE_PACKET_COUNTERS) {
	    printf("netsend_cnt      %u\n", gm_ntoh_u32 (globals->netsend_cnt));
	    printf("netrecv_cnt      %u\n", gm_ntoh_u32 (globals->netrecv_cnt));
	}
	if (GM_ENABLE_ERROR_COUNTERS) {
#define GM_ERROR_CNT(name,desc)										\
		printf ("%-40s %u\n", #name, gm_ntoh_u32 (globals->name ## _error_cnt));
		GM_ERROR_COUNTERS
#include "gm_error_counters.h"
	}

	gm_close(port);

	return (0);
}

#define GM_MAIN gm_counters
#include "gm_main.h"


/*
  This comment holds special emacs settings for this file.

  Local Variables:
  tab-width:4
  c-file-style:"bsd"
  End:
*/

==> gm-2.0.8_Linux.tar.gz: gm-2.0.8_Linux/tests/gm_counters.c <==
/******************************************************************-*-c-*-
 * Myricom GM networking software and documentation                      *
 * Copyright (c) 1999 by Myricom, Inc.                                   *
 * All rights reserved.  See the file `COPYING' for copyright notice.    *
 *************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#if defined __STDC__ || defined WIN32
#if GM_OS_VXWORKS
#include "vxWorks.h"
#endif  /* GM_OS_VXWOKRS */
#include <stdarg.h>
#else
#include <varargs.h>
#endif

/* This program uses undocumented software interfaces that are not
   intended for use in code written outside Myricom.  Hence the
   following undocumented includes. */

#include "gm_internal.h"
#include "gm_enable_error_counters.h"
#include "gm_enable_debug_counters.h"
#include "gm_enable_packet_counters.h"

/* this 1024 was hard-coded in drivers/gm.c */
/* don't really want to get all the data from a card with 2Meg+ */
#define MY_MAX_NODE	(3*1024)

/****************************************************************
 * global data
 ****************************************************************/

int board_num = 0,
    do_clear = 0,
    port_num  = 0;

#if	GM_OS_AIX

struct gm_lanai_globals *globals = NULL;

#else	/* GM_OS_AIX */

unsigned int space[(sizeof(struct gm_lanai_globals) +
		       ((MY_MAX_NODE+1) * sizeof(struct gm_connection))) / sizeof(int)];

struct gm_lanai_globals *globals = (struct gm_lanai_globals *) space;

#endif	/* GM_OS_AIX */

gm_u64_t current_rtc;

gm_port_t *port;

extern int atoi(const char *);

/****************************************************************
 * Command line processing
 ****************************************************************/

#define TP(a) fprintf (stderr, a)

static void
print_synopsis(void)
{
  TP ("\n");
  TP ("Myricom 'gm_counters' displays diagnostic statistics about the\n");
  TP ("operation of the Myrinet interface.  This program is used almost\n");
  TP ("exclusively for problem diagnosis.  The output from 'gm_counters'\n");
  TP ("is a subset of the output from 'gm_debug'.\n");
}


static void
print_options (void)
{
  TP ("\nusage: gm_counters [options]\n\n");

  TP ("-B, --board=NUM      [0] board from which to display the counters\n");
  TP ("-p, --port=NUM       [0] port number to use\n");
  TP ("-C, --clear              clears all counters after printing\n");
  TP ("    --help               output usage and options information, then exit\n");
  printf ("\n");

#undef TP
}


static void
#ifdef va_dcl
usage (fmt, va_alist)
     const char *fmt;
     va_dcl
#else
usage (const char *fmt, ...)
#endif
{
  va_list ap;

  print_options();

/* Now print the caller's diagnostic */
#if defined __STDC__ || defined WIN32
  va_start (ap, fmt);
#else
  va_start (ap);
#endif
  vfprintf (stderr, fmt, ap);
  va_end (ap);

} /* usage() */


static void
parse_args (int argc, char *argv[])
{
/* NOTE: If you change the operating characteristics or the flags
 *       used by this program, please notify Max so he can change the
 *       Myrinet Handbook document.
 */
  int i;

  /* The 'F' macro is for all flags with no argument (parameter)     */
#define F(name,action) if (strcmp (argv[i], name) == 0) { action; continue; }

  for (i = 1; i < argc; i++)
  {
      /* Dispose of the --help flag first                                */
      if (strcmp (argv[i], "--help") == 0)
	  {
		  print_synopsis();
		  print_options();
		  gm_exit (GM_SUCCESS);
	  }

      /* Here are the flags with no argument.                            */
      /* Please keep short form, if any, after its long form equivalent, */
	  /* but keep the long form options in alpha sequence.               */

      /* None in gm_counters at present */

#undef F

      /* The check_opt macro is for long-form options with numeric        */
      /* arguments and simple validation tests.                           */
#define check_opt(prefix, var, test)							\
	  if (sscanf (argv[i], "--" prefix "=%i", &var) == 1)		\
	  {															\
	      if (!(test))											\
		  {														\
			  usage ("bad value for option \"%s\"\n", argv[i]);	\
			  gm_exit (GM_FAILURE);								\
		  }														\
	      continue;												\
	  }

      /* Here are the flags with arguments, sequenced as above */

      check_opt ("board", board_num, board_num >= 0);
      if (strcmp (argv[i], "-B") == 0)
	  {
		  if (i + 1 >= argc)
		  {
			  usage ("Board number expected after '-B'.\n");
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  board_num = atoi (argv[++i]);
		  if (board_num < 0)
		  {
			  usage ("bad board number: %d\n", board_num);
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  continue;
	  }

      /* Clearing counters after printing ? */
      if (strcmp (argv[i], "-C") == 0 ||
	  strcmp (argv[i], "--clear") == 0)
	{
	  do_clear = 1;
	  continue;
	}
     

      /* Note that at present the parameter to gm_num_ports is a dummy */
      check_opt ("port", port_num,
				 (port_num >= 0 && port_num < gm_num_ports ((gm_port_t *)0)));
      if (strcmp (argv[i], "-p") == 0)
	  {
		  if (i + 1 >= argc)
		  {
			  usage ("Port number expected after '-p'.\n");
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  port_num = atoi (argv[++i]);
		  if (port_num < 0 || port_num >= gm_num_ports ((gm_port_t *)0))
		  {
			  usage ("bad port number: %d\n", port_num);
			  gm_exit (GM_INVALID_PARAMETER);
		  }
		  continue;
	  }

#undef check_opt

      usage ("unknown arg: '%s'\n", argv[i]);
      gm_exit (GM_FAILURE);
  }
}



/****************************************************************
 * produce the actual output
 ****************************************************************/

static int
gm_counters(int argc, char **argv)
{
	int length;
	gm_status_t status;

	parse_args(argc, argv);

	printf("gm_counters: Opening board %d port %d\n", board_num, port_num);
	fflush(stdout);				/* just in case we hang in the open */

	status = gm_open (&port, board_num, port_num, "gm_counters",
					  (enum gm_api_version) GM_API_VERSION);
	if (status != GM_SUCCESS) {
		gm_perror("gm_counters: gm_open() failed", status);
		gm_exit(status);
	}

	length = sizeof(struct gm_lanai_globals) +
	    (MY_MAX_NODE * sizeof(struct gm_connection));

#if	GM_OS_AIX
	if ((globals = malloc (length)) == NULL) {
		printf ("gm_counters: error in malloc\n");
		gm_exit (GM_FAILURE);
	}
#endif	/* GM_OS_AIX */

	status = _gm_get_globals(port, (gm_u8_t *) globals, length);
	if (status != GM_SUCCESS) {
		gm_perror ("failed to get LANai globals", status);
		gm_exit(status);
	}

	/* now we have a copy of the lanai_globals */

	if (GM_ENABLE_PACKET_COUNTERS) {
	    printf("netsend_cnt      %u\n", gm_ntoh_u32 (globals->netsend_cnt));
	    printf("netrecv_cnt      %u\n", gm_ntoh_u32 (globals->netrecv_cnt));
	}
	if (GM_ENABLE_ERROR_COUNTERS) {
#define GM_ERROR_CNT(name,desc)										\
		printf ("%-40s %u\n", #name, gm_ntoh_u32 (globals->name ## _error_cnt));
		GM_ERROR_COUNTERS
#include "gm_error_counters.h"
	}

	/* Clear counters if requested */
	if (do_clear)
	  {
	    status = _gm_clear_counters(port);
	    if (status != GM_SUCCESS)
	      {
		gm_perror("Error clearing counters", status);
		gm_exit(GM_FAILURE);
	      }
	  }

	gm_close(port);

	return (0);
}

#define GM_MAIN gm_counters
#include "gm_main.h"


/*
  This comment holds special emacs settings for this file.

  Local Variables:
  tab-width:4
  c-file-style:"bsd"
  End:
*/
